#include <iostream>
#include <iomanip>
#include <math.h>

#ifndef __R3_HH__
#define __R3_HH__

using namespace std;

class R3{
private:
  double x[3];

public:

  static const int X = 0;  static const int Y = 1;  static const int Z = 2;

  R3(){x[0]=x[1]=x[2]=0.0;}
  R3(double x0, double x1, double x2){x[0]=x0;  x[1]=x1;  x[2]=x2;}

  double norm() const {return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);}

  operator       double* ()        {return x;}
  operator const double* ()  const {return x;}
  
  double operator [] (int i) const {return x[i];}
  
  friend R3     operator * (double s, const R3& r)
  {return R3(r[0]*s, r[1]*s, r[2]*s);}
  friend R3     operator * (const R3& r, double s) {return s*r;}
  friend R3     operator / (const R3& r, double s) {return (1.0/s)*r;}
  
  friend R3     operator + (const R3& r1, const R3& r2)
  {return R3(r1[0]+r2[0], r1[1]+r2[1], r1[2]+r2[2]);}
  
  friend R3     operator - (const R3& r1, const R3& r2)
  {return R3(r1[0]-r2[0], r1[1]-r2[1], r1[2]-r2[2]);}
  // cross product
  friend R3     operator ^ (const R3& r1, const R3& r2)
  {return R3(r1[1]*r2[2] - r1[2]*r2[1], 
	     r1[2]*r2[0] - r1[0]*r2[2], 
	     r1[0]*r2[1] - r1[1]*r2[0]);}
  
  friend double operator * (const R3& r1, const R3& r2)
  {return r1[0]*r2[0] + r1[1]*r2[1] + r1[2]*r2[2];}

  friend ostream& operator <<  (ostream& s, const R3& r){
    int p;
    p = cout.precision();
    cout.precision(12);
    cout.setf(ios::fixed, ios::floatfield);
    cout << r[0] << " " << r[1] << " " << r[2];
    cout.precision(p);
    return s;
  }
};


#endif
