#include "SE3.hh"

SE3::operator so3() const{
  double theta = acos( (H[0][0]+H[1][1]+H[2][2] - 1.0)/2);

  if(so3::EPSILON < fabs(theta))
    return so3( R3( (H[2][1]-H[1][2])/(2.0*sin(theta)),
		    (H[0][2]-H[2][0])/(2.0*sin(theta)),
		    (H[1][0]-H[0][1])/(2.0*sin(theta)) ), theta );
  else return so3(R3(), theta); 
}

SE3 operator * (const SE3& h1, const SE3& h2){
  SE3 h3;
  double *ptr = (double*)h3;

  *ptr++ = (h1[SE3::R11]*h2[SE3::R11] + 
	    h1[SE3::R12]*h2[SE3::R21] + 
	    h1[SE3::R13]*h2[SE3::R31]); 

  *ptr++ = (h1[SE3::R11]*h2[SE3::R12] + 
	    h1[SE3::R12]*h2[SE3::R22] + 
	    h1[SE3::R13]*h2[SE3::R32]); 

  *ptr++ = (h1[SE3::R11]*h2[SE3::R13] + 
	    h1[SE3::R12]*h2[SE3::R23] + 
	    h1[SE3::R13]*h2[SE3::R33]); 

  *ptr++ = (h1[SE3::R11]*h2[SE3::TX] + 
	    h1[SE3::R12]*h2[SE3::TY] + 
	    h1[SE3::R13]*h2[SE3::TZ] + h1[SE3::TX]); 

  *ptr++ = (h1[SE3::R21]*h2[SE3::R11] + 
	    h1[SE3::R22]*h2[SE3::R21] + 
	    h1[SE3::R23]*h2[SE3::R31]); 

  *ptr++ = (h1[SE3::R21]*h2[SE3::R12] + 
	    h1[SE3::R22]*h2[SE3::R22] + 
	    h1[SE3::R23]*h2[SE3::R32]); 

  *ptr++ = (h1[SE3::R21]*h2[SE3::R13] + 
	    h1[SE3::R22]*h2[SE3::R23] + 
	    h1[SE3::R23]*h2[SE3::R33]); 

  *ptr++ = (h1[SE3::R21]*h2[SE3::TX] + 
	    h1[SE3::R22]*h2[SE3::TY] + 
	    h1[SE3::R23]*h2[SE3::TZ] + h1[SE3::TY]); 

  *ptr++ = (h1[SE3::R31]*h2[SE3::R11] + 
	    h1[SE3::R32]*h2[SE3::R21] + 
	    h1[SE3::R33]*h2[SE3::R31]); 

  *ptr++ = (h1[SE3::R31]*h2[SE3::R12] + 
	    h1[SE3::R32]*h2[SE3::R22] + 
	    h1[SE3::R33]*h2[SE3::R32]); 

  *ptr++ = (h1[SE3::R31]*h2[SE3::R13] + 
	    h1[SE3::R32]*h2[SE3::R23] + 
	    h1[SE3::R33]*h2[SE3::R33]); 

  *ptr++ = (h1[SE3::R31]*h2[SE3::TX] + 
	    h1[SE3::R32]*h2[SE3::TY] + 
	    h1[SE3::R33]*h2[SE3::TZ] + h1[SE3::TZ]); 

  return h3;
}

SE3 operator ^ (const SE3& E, int i){
  if(i == -1){
    R3 p = -1.0*((R3)E);
    SO3 Rt = !((SO3)E);
    return SE3(Rt, Rt*p); 
  }
  return SE3();
}

ostream& operator <<  (ostream& s, const SE3& h){
  int p;
  p = s.precision();
  s.precision(12);
  s.setf(ios::fixed, ios::floatfield);
  s.setf(ios::right, ios::adjustfield);

  s << setw(17) << h[SE3::R11] 
    << setw(17) << h[SE3::R12] 
    << setw(17) << h[SE3::R13]
    << setw(17) << h[SE3::TX]  << endl
    << setw(17) << h[SE3::R21] 
    << setw(17) << h[SE3::R22] 
    << setw(17) << h[SE3::R23]
    << setw(17) << h[SE3::TY]  << endl
    << setw(17) << h[SE3::R31] 
    << setw(17) << h[SE3::R32] 
    << setw(17) << h[SE3::R33]
    << setw(17) << h[SE3::TZ]  << endl
    << setw(17) << 0 
    << setw(17) << 0 
    << setw(17) << 0
    << setw(17) << 1;

  s.setf(ios::left, ios::adjustfield);
  s.precision(p);
  return s;
}

