#include "SO3.hh"
#include "so3.hh"

SO3::operator so3() const{
  double theta = acos( (R[0][0]+R[1][1]+R[2][2] - 1.0)/2);

  if(so3::EPSILON < fabs(theta)){
    return so3(R3( (R[2][1]-R[1][2])/(2.0*sin(theta)),
		   (R[0][2]-R[2][0])/(2.0*sin(theta)),
		   (R[1][0]-R[0][1])/(2.0*sin(theta)) ), theta);
  }
  else return so3(R3(), theta); 
}

SO3 operator * (const SO3& r1, const SO3& r2){
  SO3 R;

  double* r = (double*)R;

  *r++ = (r1[SO3::R11]*r2[SO3::R11] +
	  r1[SO3::R12]*r2[SO3::R21] +
	  r1[SO3::R13]*r2[SO3::R31]);

  *r++ = (r1[SO3::R11]*r2[SO3::R12] +
	  r1[SO3::R12]*r2[SO3::R22] +
	  r1[SO3::R13]*r2[SO3::R32]);

  *r++ = (r1[SO3::R11]*r2[SO3::R13] +
	  r1[SO3::R12]*r2[SO3::R23] +
	  r1[SO3::R13]*r2[SO3::R33]);

  *r++ = (r1[SO3::R21]*r2[SO3::R11] +
	  r1[SO3::R22]*r2[SO3::R21] +
	  r1[SO3::R23]*r2[SO3::R31]);

  *r++ = (r1[SO3::R21]*r2[SO3::R12] +
	  r1[SO3::R22]*r2[SO3::R22] +
	  r1[SO3::R23]*r2[SO3::R32]);

  *r++ = (r1[SO3::R21]*r2[SO3::R13] +
	  r1[SO3::R22]*r2[SO3::R23] +
	  r1[SO3::R23]*r2[SO3::R33]);

  *r++ = (r1[SO3::R31]*r2[SO3::R11] +
	  r1[SO3::R32]*r2[SO3::R21] +
	  r1[SO3::R33]*r2[SO3::R31]);

  *r++ = (r1[SO3::R31]*r2[SO3::R12] +
	  r1[SO3::R32]*r2[SO3::R22] +
	  r1[SO3::R33]*r2[SO3::R32]);

  *r   = (r1[SO3::R31]*r2[SO3::R13] +
	  r1[SO3::R32]*r2[SO3::R23] +
	  r1[SO3::R33]*r2[SO3::R33]);

  return R;
}

ostream& operator <<  (ostream& s, const SO3& r){
  int p;
  p = cout.precision();
  cout.precision(12);
  cout.setf(ios::fixed, ios::floatfield);
  cout.setf(ios::right, ios::adjustfield);
  cout << setw(17) << r[SO3::R11] 
       << setw(17) << r[SO3::R12] 
       << setw(17) << r[SO3::R13] << endl
       << setw(17) << r[SO3::R21] 
       << setw(17) << r[SO3::R22] 
       << setw(17) << r[SO3::R23] << endl
       << setw(17) << r[SO3::R31] 
       << setw(17) << r[SO3::R32] 
       << setw(17) << r[SO3::R33];
  cout.setf(ios::left, ios::adjustfield);
  cout.precision(p);
  return s;
}

